"use strict";
var es6_promise_1 = require("es6-promise");
var Store = (function () {
    function Store(initialState, cloner, notifier) {
        this.cloner = cloner;
        this.notifier = notifier;
        this.state = this.cloner.clone(initialState);
    }
    Store.prototype.cloneState = function () {
        return this.cloner.clone(this.state);
    };
    Store.prototype.updateState = function (state) {
        this.state = this.cloner.clone(state);
        this.notifier.notify();
    };
    Store.prototype.dispatch = function (action) {
        var state = action.execute(this.cloneState());
        this.updateState(state);
        return state;
    };
    Store.prototype.dispatchAsync = function (action) {
        var _this = this;
        var promise = new es6_promise_1.Promise(function (resolve, reject) {
            action.execute(_this.cloneState(), resolve, reject);
        });
        promise.then(function (state) {
            _this.updateState(state);
        });
        return promise;
    };
    Store.prototype.getState = function () {
        return this.cloner.clone(this.state);
    };
    return Store;
}());
exports.Store = Store;
